import numpy as np
from scipy.stats import gaussian_kde
from matplotlib import gridspec
import matplotlib.pyplot as plt
import corner
from seaborn.palettes import SEABORN_PALETTES
import os
from pesummary.core.plots.bounded_1d_kde import bounded_1d_kde
colorblind = SEABORN_PALETTES["colorblind"]

base_105 = "/home/rory.smith/projects/O3/S200105ae/C01_suppressed_psd/"
downsample_105 = "/home/soichiro.morisaki/pe_runs/O3b/S200105ae_S200115j/downsampled_production/S200105ae"
result_file_map_GW200105_lowspin = {
    "PhenomNSBH": "/home/andrew.matas/public_html/projects/O3/PE/S200105ae/IMRPhenomNSBH_C01_Production_mcq/samples/IMRPhenomNSBH_pesummary.dat",
    "PhenomNSBH (no tides)": "/home/andrew.matas/public_html/projects/O3/PE/S200105ae/IMRPhenomNSBH_C01_Production_mcq_lambda2_0/samples/IMRPhenomNSBH_pesummary.dat",  # review pending (01/30/2021)
    "PhenomXAS": "/home/rory.smith/public_html/O3/S200105ae/C01_suppressed_psd/mc_q_production/XAS/lowspin/samples/lowspin_pesummary.dat",
    "PhenomXAS(a1<0.5)": os.path.join(downsample_105, "PhenomXAS.dat"),
    "PhenomXHM": "/home/rory.smith/public_html/O3/S200105ae/C01_suppressed_psd/mc_q_production/XHM/lowspin/samples/lowspin_pesummary.dat",
    "PhenomXHM(a1<0.5)": os.path.join(downsample_105, "PhenomXHM.dat"),
    "PhenomXPHM": "/home/rory.smith/public_html/O3/S200105ae/C01_suppressed_psd/mcq_production/XPHM/lowspin/samples/lowspin_pesummary.dat",
    "PhenomXPHM(a1<0.5)": os.path.join(downsample_105, "PhenomXPHM.dat"),
    "PhenomXPHM_LI": "/home/soichiro.morisaki/public_html/S200105ae/IMRPhenomXPHMpseudoFourPN_precessingBBH_lowspin_mcmc_deglitched_V1suppressedPSD/prelim210104/without_bad_chains/samples/IMRPhenomXPHMpseudoFourPN_pesummary.dat",
    "PhenomXPHM_bugfix": "/home/virginia.demilio/public_html/O3_runs/S200105ae/PHM-all/samples/Bilby-XPHM-low-spin_pesummary.dat",
    "PhenomXPHM_combined": "/home/leo.tsukada/projects/O3_NSBH_pe/S200105ae/pe_repo/combined_samples/pbilby_rift_PHM_lowspin_combined.dat",
    "SEOBNRv4NSBH": "/home/andrew.matas/public_html/projects/O3/PE/S200105ae/SEOBNRv4_ROM_NRTidalv2_NSBH_C01_Production_mcq/samples/SEOBNRv4_ROM_NRTidalv2_NSBH_pesummary.dat",
    "SEOBNRv4NSBH(a1<0.5)": os.path.join(downsample_105, "SEOBNRv4NSBH.dat"), 
    "SEOBNRv4NSBH_notides": "/home/andrew.matas/public_html/projects/O3/PE/S200105ae/SEOBNRv4_ROM_NRTidalv2_NSBH_C01_Production_mcq_lambda2_0/samples/SEOBNRv4_ROM_NRTidalv2_NSBH_lambda2-0_pesummary.dat",
    "SEOBNRv4NSBH_notides(a1<0.5)": os.path.join(downsample_105, "SEOBNRv4NSBH_notides.dat"),
    "prior": "/home/soichiro.morisaki/public_html/S200105ae/prior_precessingBBH_lowspin/1262276684.0572083-359417/pesummary/samples/1596809631_posterior_V1L1_1262276684_0572083-359417_pesummary.dat",
    "SEOBNRv4": "/home/luca.prudenzi/public_html/O3/S200105ae/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4_ROM_lowspin_pesummary.dat",  # review pending (01/30/2021)
    "SEOBNRv4(a1<0.5)": os.path.join(downsample_105, "SEOBNRv4.dat"),
    "SEOBNRv4HM": "/home/luca.prudenzi/public_html/O3/S200105ae/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4HM_ROM_lowspin_pesummary.dat",  # review pending (01/30/2021)
    "SEOBNRv4HM(a1<0.5)": os.path.join(downsample_105, "SEOBNRv4HM.dat"),
    "SEOBNRv4PHM": "/home/anjali.yelikar/public_html/O3_PE_local/GW200105/C01_deglitched-lowspin-v8_PHM/samples/C01-deglitched-RIFT-EOBv4PHM_pesummary.dat",
    "SEOBNRv4PHM(a1<0.5)": os.path.join(downsample_105, "SEOBNRv4PHM.dat")
}
result_file_map_GW200105_highspin = {
    "PhenomXAS": os.path.join(base_105, "XAS/samples/high_spin_pesummary.dat"),  # m1-m2 sampling
    "PhenomXAS_LI": "/home/soichiro.morisaki/public_html/S200105ae/IMRPhenomXASpseudoFourPN_alignedBBH_highspin_nest/pesummary/samples/PhenomXAS_highspin_pesummary.dat",
    "PhenomXHM": "/home/rory.smith/public_html/O3/S200105ae/C01_suppressed_psd/mc_q_production/XHM/highspin/samples/XHM_highspin_pesummary.dat",
    "PhenomXPHM":  "/home/rory.smith/public_html/O3/S200105ae/C01_suppressed_psd/mcq_production/XPHM/highspin/samples/XPHM_highspin_pesummary.dat",
    "PhenomXPHM_bugfix": "/home/virginia.demilio/public_html/O3_runs/S200105ae/PHM-all/samples/Bilby-XPHM-high-spin_pesummary.dat",
    "PhenomXPHM_LI": "/home/soichiro.morisaki/public_html/S200105ae/IMRPhenomXPHMpseudoFourPN_precessingBBH_highspin_mcmc_deglitched_V1suppressedPSD/prelim210104/without_bad_chains/samples/IMRPhenomXPHMpseudoFourPN_pesummary.dat",
    "PhenomXPHM_combined": "/home/leo.tsukada/projects/O3_NSBH_pe/S200105ae/pe_repo/combined_samples/pbilby_rift_PHM_highspin_combined.dat",
    "prior": "/home/soichiro.morisaki/public_html/S200105ae/prior_precessingBBH_highspin/1262276684.0572083-359417/pesummary/samples/1596809578_posterior_V1L1_1262276684_0572083-359417_pesummary.dat",
    "SEOBNRv4": "/home/luca.prudenzi/public_html/O3/S200105ae/SEOBNRv4_ROM/samples/SEOBNRv4_ROM_highspin_pesummary.dat",
    "SEOBNRv4HM": "/home/luca.prudenzi/public_html/O3/S200105ae/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4HM_ROM_highspin_pesummary.dat",  # review pending (01/30/2021)
    "SEOBNRv4PHM": "/home/anjali.yelikar/public_html/O3_PE_local/GW200105/C01_deglitched-highspin-v8/samples/C01-deglitched-RIFT-EOBv4PHM_pesummary.dat"
}

base_115 = "/home/rory.smith/public_html/O3/S200115j"
downsample_115 = "/home/soichiro.morisaki/pe_runs/O3b/S200105ae_S200115j/downsampled_production/S200115j"
result_file_map_GW200115_lowspin = {

    "PhenomNSBH": "/home/andrew.matas/public_html/projects/O3/PE/S200115j/IMRPhenomNSBH_C01_Production_mcq/samples/IMRPhenomNSBH_pesummary.dat",
    "PhenomNSBH (no tides)": "/home/andrew.matas/public_html/projects/O3/PE/S200115j/IMRPhenomNSBH_C01_Production_mcq_lambda2-0/samples/IMRPhenomNSBH_lambda2-0_pesummary.dat",
    "PhenomXAS": "/home/rory.smith/public_html/O3/S200115j/C01_suppressed_V1/mc_q_production/XAS/lowspin/samples/lowspin_pesummary.dat",
    "PhenomXAS(a1<0.5)": os.path.join(downsample_115, "PhenomXAS.dat"),
    "PhenomXHM": "/home/rory.smith/public_html/O3/S200115j/C01_suppressed_V1/XHM/mc_q_production/lowspin/samples/lowspin_pesummary.dat",
    "PhenomXHM(a1<0.5)": os.path.join(downsample_115, "PhenomXHM.dat"),
    "PhenomXPHM": "/home/virginia.demilio/projects/O3/S200115j/outdir_IMRPhenomXPHM_C01_BBH_precession_NSconstraint_a/result/S200115j_C01_IMRPhenomXPHM_BBH_precession_NSconstraint_a_0_samples_ra_and_dec.dat",
    "PhenomXPHM(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM.dat"),
    "PhenomXPHM_nobugfix": "/home/rory.smith/public_html/O3/S200115j/C01_suppressed_V1/XPHM/soichiroprior/xphm_bugfix/samples/pbilby_nobugfix_pesummary.dat",
    "PhenomXPHM_nobugfix(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM_nobugfix.dat"),
    # "PhenomXPHM": "/home/virginia.demilio/projects/O3/S200115j/C01/outdir_IMRPhenomXPHM_C01_BBH_precession_NSconstraint_a/result/S200115j_C01_IMRPhenomXPHM_BBH_precession_NSconstraint_a_0_result.json"
    "PhenomXPHM_LI": "/home/soichiro.morisaki/public_html/S200115j/IMRPhenomXPHMpseudoFourPN_precessingBBH_lowspin_mcmc_quick/prelim210104/samples/IMRPhenomXPHMpseudoFourPN_pesummary.dat",
    "PhenomXPHM_LI(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM_LI.dat"),
    "PhenomXPHM_RIFTv3rf": "/home/anjali.yelikar/public_html/O3_PE_local/GW200115/C01-lowspin-PhenomXPHM/samples/C01-RIFT-XPHM-lowspin-v3-rf_pesummary.dat",
    "PhenomXPHM_RIFTv3rf(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM_RIFTv3rf.dat"),
    "PhenomXPHM_RIFTv1gp": "/home/anjali.yelikar/public_html/O3_PE_local/GW200115/C01-lowspin-PhenomXPHM/samples/C01-RIFT-XPHM-lowspin-v1-gp_pesummary.dat",
    "PhenomXPHM_RIFTv1gp(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM_RIFTv1gp.dat"),
    "PhenomXPHM_bugfix": "/home/rory.smith/public_html/O3/S200115j/C01_suppressed_V1/XPHM/soichiroprior/xphm_bugfix/samples/xphm_bugfix_pesummary.dat",
    "PhenomXPHM_bugfix(a1<0.5)": os.path.join(downsample_115, "PhenomXPHM_bugfix.dat"),
    "SEOBNRv4NSBH" : "/home/andrew.matas/public_html/projects/O3/PE/S200115j/SEOBNRv4_ROM_NRTidalv2_NSBH_C01_Production_mcq/samples/SEOBNRv4_ROM_NRTidalv2_NSBH_pesummary.dat",
    "SEOBNRv4NSBH(a1<0.5)": os.path.join(downsample_115, "SEOBNRv4NSBH.dat"),
    "SEOBNRv4NSBH_notides" : "/home/andrew.matas/public_html/projects/O3/PE/S200115j/SEOBNRv4_ROM_NRTidalv2_NSBH_C01_Production_mcq_lambda2-0/samples/SEOBNRv4_ROM_NRTidalv2_NSBH_lambda2-0_pesummary.dat",
    "SEOBNRv4NSBH_notides(a1<0.5)" : os.path.join(downsample_115, "SEOBNRv4NSBH_notides.dat"),
    "SEOBNRv4": "/home/luca.prudenzi/public_html/O3/S200115j/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4_ROM_lowspin_pesummary.dat",
    "SEOBNRv4(a1<0.5)": os.path.join(downsample_115, "SEOBNRv4.dat"),
    "SEOBNRv4HM": "/home/luca.prudenzi/public_html/O3/S200115j/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4HM_ROM_lowspin_pesummary.dat",
    "SEOBNRv4HM(a1<0.5)": os.path.join(downsample_115, "SEOBNRv4HM.dat"),
    "SEOBNRv4PHM": "/home/anjali.yelikar/public_html/O3_PE_local/GW200115/C01-lowspin-v5/samples/C01-RIFT-v4PHM_pesummary.dat",
    "SEOBNRv4PHM(a1<0.5)": os.path.join(downsample_115, "SEOBNRv4PHM.dat")
}
result_file_map_GW200115_highspin = {
    "PhenomXAS": "/home/virginia.demilio/public_html/O3_runs/S200115j/C01/final_runs/XAS-highspin/samples/C01-bilby-XAS-highspin_pesummary.dat",
    "PhenomXHM": "/home/virginia.demilio/public_html/O3_runs/S200115j/C01/final_runs/XHM-highspin/samples/C01-bilby-XHM-highspin_pesummary.dat",
#    "PhenomXPHM": os.path.join(base_115, "C01_suppressed_V1/XPHM/highspin/samples/XPHM_highspin_pesummary.dat"),
    "PhenomXPHM_nact75": "/home/virginia.demilio/public_html/O3_runs/S200115j/C01/final_runs/XPHM_nact75_comparison/samples/C01-Bilby-PhenomXPHM-mcq_pesummary.dat",
    "PhenomXPHM_RIFT": "/home/anjali.yelikar/public_html/O3_PE_local/GW200115/C01-highspin-PhenomXPHM/samples/C01-RIFT-PhenomXPHM-intrinsic-prelim_pesummary.dat",
    # "PhenomXPHM_cecilio": "/home/cecilio.garcia-quiros/projects/O3/PE/S200115j/series1X_S200115j/S1X_S200115j_IMRPhenomXPHM_PV223_FS3_XPMB0.1_XHMMB0.1_N2048_NA30_Dmarg/result/S1X_S200115j_IMRPhenomXPHM_PV223_FS3_XPMB0.1_XHMMB0.1_N2048_NA30_Dmarg_0_samples.dat",
    "PhenomXPHM_cecilio": "/home/virginia.demilio/public_html/O3_runs/S200115j/C01/final_runs/XPHM_as_of_Jan13/samples/bilby-xphm-highspin_pesummary.dat",
    "PhenomXPHM_LI": "/home/soichiro.morisaki/public_html/S200115j/IMRPhenomXPHMpseudoFourPN_precessingBBH_highspin_mcmc_quick/prelim210104/samples/IMRPhenomXPHMpseudoFourPN_pesummary.dat",
    "SEOBNRv4HM": "/home/luca.prudenzi/public_html/O3/S200115j/Prod_final_v4low_v4HMlow_v4HMhigh/samples/SEOBNRv4HM_ROM_highspin_pesummary.dat",
    "SEOBNRv4PHM": "/home/anjali.yelikar/public_html/O3_PE_local/GW200115/C01-highspin-v5/samples/C01-RIFT-v4PHM-highspin_pesummary.dat"
#     "PhenomXPHM": os.path.join(base_115, "C01_C00cal/XPHM/samples/high_spin_pesummary.dat")
}

result_file_map_GW190426 = {
    "Tv2NSBH" : "/home/leo.tsukada/paper_material/gw200105-gw200115-discovery/scripts/plotting/posterior_samples_from_CIT/GW190426_posterior_samples.h5"
}

result_file_map_GW190814 = {
    "PHMcombined" : "/home/leo.tsukada/paper_material/gw200105-gw200115-discovery/scripts/plotting/posterior_samples_from_CIT/GW190814_posterior_samples.h5"
}

color_dict = {
    "GW200105": colorblind[3],
    "GW200115": colorblind[0],
    "Prior": colorblind[7]
}

linestyle_dict = {
    "PhenomPv3HM": "-",
    "SEOBNRv4PHM": "-",
    "PhenomXHM": "--",
    "PhenomXPHM": "-",
    "PhenomXPHM_LI": "-",
    "SEOBNRv4HM": "--",
    "PhenomPv3": ":",
    "SEOBNRv4P": ":",
    "PhenomD": "-.",
    "SEOBNRv4": "-.",
    "PhenomNSBH": "-",
    "SEOBNRv4NSBH": "-",
    "combined_PHM": "-",
}

alpha_dict = {
    "PhenomPv3HM": 0.3,
    "SEOBNRv4PHM": 0.3,
    "PhenomXHM": 0.3,
    "PhenomXPHM": 0.3,
    "PhenomXPHM_LI": 0.3,
    "SEOBNRv4HM": 0.3,
    "PhenomPv3": 0.3,
    "SEOBNRv4P": 0.3,
    "PhenomD": 0.3,
    "SEOBNRv4": 0.3,
    "PhenomNSBH": 0.3,
    "SEOBNRv4NSBH": 0.3,
    "combined_PHM": 0.3,
}

label_dict = {
    "PhenomPv3HM": "Phenom PHM",
    "SEOBNRv4PHM": "EOBNR PHM",
    "PhenomHM": "Phenom HM",
    "PhenomXAS": "Phenom XAS",
    "PhenomXHM": "Phenom XHM",
    "PhenomXPHM": "Phenom XPHM",
    "PhenomXPHM_LI": "Phenom XPHM",
    "SEOBNRv4HM": "EOBNR HM",
    "PhenomPv3": "Phenom P",
    "SEOBNRv4P": "EOBNR P",
    "PhenomD": "Phenom",
    "SEOBNRv4": "EOBNR",
    "PhenomNSBH": "Phenom NSBH",
    "SEOBNRv4NSBH": "EOBNR NSBH",
    "combined_PHM": "Combined PHM",
}

linewidth_dict = {
    "PhenomPv3HM": 1.5,
    "SEOBNRv4PHM": 1.5,
    "PhenomXHM": 1.5,
    "PhenomXPHM": 1.5,
    "PhenomXPHM_LI": 1.5,
    "SEOBNRv4HM": 1.5,
    "PhenomPv3": 1.5,
    "SEOBNRv4P": 1.5,
    "PhenomD": 1.5,
    "SEOBNRv4": 1.5,
    "PhenomNSBH": 1.5,
    "SEOBNRv4NSBH": 1.5,
    "combined_PHM": 2.5,
}


def add_percentile_lines(
    percentiles, ax, ptype="part", color=None, lw=None, ls=None, zorder=None,
    swap_axis=False, right_figure=False
):
    function = ax.axvline
    if swap_axis:
        function = ax.axhline
    for pp in percentiles:
        if ptype == "part":
            if right_figure:
                lim = {"xmax": 0.05}
            else:
                lim = {"ymax": 0.05}
            function(pp, color=color, lw=lw, ls=ls, zorder=zorder, **lim)
        elif ptype == "full":
            function(pp, color=color, lw=lw, ls=ls, zorder=zorder)
        elif isinstance(ptype, float):
            if right_figure:
                lim = {"xmax": ptype}
            else:
                lim = {"ymax": ptype}
            function(pp, color=color, lw=lw, ls=ls, zorder=zorder, **lim)


def add_samples_to_fig(
    xx, xxlims, label, alpha, ls, c, ax, zorder=-10, weights=1., fill=True, right_figure=False,
    swap_axis=False, ptype="part", linewidth=2, line_alpha=1.0, kdemethod=None, smooth=3., add_ci=True
):
    xlow, xhigh = xxlims
    xsmooth = np.linspace(xlow, xhigh, 1000)

    lw = linewidth
    if kdemethod is None:
        kde = gaussian_kde(xx)
    elif kdemethod=="Reflection":
        kde = bounded_1d_kde(xx, xlow=xlow, xhigh=xhigh)
    elif kdemethod=="Transform":
        kde = bounded_1d_kde(xx, xlow=xlow, xhigh=xhigh, method="Transform", apply_smoothing=True, smooth=smooth)
    else:
        print("method should be Reflection or Transform!")
        raise
    if kdemethod=="Transform":
        xsmooth, pdfs = kde(xsmooth)
    else:
        pdfs = kde(xsmooth)
    try:
        percentiles = xx.confidence_interval([5, 95])
    except AttributeError:
        percentiles = [np.percentile(xx, i) for i in [5, 95]]
    if not swap_axis:
        ax.plot(
            xsmooth, weights * pdfs, color=c, lw=lw, ls=ls, label=label,
            zorder=zorder, alpha=line_alpha
        )
        if add_ci:
            add_percentile_lines(
                percentiles, ax, ptype=ptype, color=c, lw=2.0, ls=ls,
                zorder=zorder, right_figure=right_figure
            )
    else:
        ax.plot(
            weights * pdfs, xsmooth, color=c, lw=lw, ls=ls, label=label,
            zorder=zorder, alpha=line_alpha
        )
        if add_ci:
            add_percentile_lines(
                percentiles, ax, ptype=ptype, color=c, lw=2.0, ls=ls, zorder=zorder,
                swap_axis=True, right_figure=right_figure
            )
    if fill and not swap_axis:
        ax.fill_between(
            xsmooth, 0, weights * pdfs, color=c, alpha=alpha, zorder=zorder
        )
    elif fill:
        ax.fill_betweenx(
            xsmooth, 0, weights * pdfs, color=c, alpha=alpha, zorder=zorder
        )


def triangle_plot_2d_axes(
    xbounds, ybounds, figsize=(8, 8),
    width_ratios=[4, 1], height_ratios=[1, 4], wspace=0.0, hspace=0.0,
    grid=False):
    """Initialize the axes for a 2d triangle plot.
    """
    high1d = 1.05

    fig = plt.figure(figsize=figsize)
    gs = gridspec.GridSpec(2, 2, width_ratios=width_ratios, height_ratios=height_ratios,
                           wspace=wspace, hspace=hspace)
    #ax3 = plt.subplot(gs[2])
    #ax1 = plt.subplot(gs[0], sharex=ax3)
    #ax4 = plt.subplot(gs[3], sharey=ax3)
    ax3 = plt.subplot(gs[2])
    ax1 = plt.subplot(gs[0])
    ax4 = plt.subplot(gs[3])

    ax1.minorticks_on()
    ax3.minorticks_on()
    ax4.minorticks_on()

    if grid:
        #ax1.grid(linestyle=':', alpha=0.5, linewidth=0.5, which='both')
        #ax3.grid(linestyle=':', alpha=0.5, linewidth=0.5, which='both')
        #ax4.grid(linestyle=':', alpha=0.5, linewidth=0.5, which='both')
        ax1.grid(which='major', ls='-')
        ax1.grid(which='minor', ls=':')
        ax3.grid(which='major', ls='-')
        ax3.grid(which='minor', ls=':')
        ax4.grid(which='major', ls='-')
        ax4.grid(which='minor', ls=':')

    # Get rid of tick labels
    ax1.xaxis.set_ticklabels([])
    ax4.yaxis.set_ticklabels([])

    # Use consistent x-axis and y-axis bounds in all 3 plots
    ax1.set_ylim(0, high1d)
    ax1.set_xlim(xbounds[0], xbounds[1])
    ax3.set_xlim(xbounds[0], xbounds[1])
    ax3.set_ylim(ybounds[0], ybounds[1])
    ax4.set_xlim(0, high1d)
    ax4.set_ylim(ybounds[0], ybounds[1])

    return fig, ax1, ax3, ax4


def create_triangular_fig_and_axes(xbounds, ybounds, figsize=(9.7, 9.7)):
    fig, ax1, ax3, ax4 = triangle_plot_2d_axes(
        xbounds, ybounds, figsize=figsize, width_ratios=[4, 1],
        height_ratios=[1, 4], wspace=0.0, hspace=0.0)
    return fig, ax1, ax3, ax4


def add_samples_to_triangular_fig(
    samples, event, waveform_model, xlims, ylims, zorder, ax1, ax3, ax4, ptype="full",
    plot_density=True, add_ci_list=[True, True], label=None,
    fill_contours=True, no_fill_contours=False, fill_histogram=True,
    ls=None, lw=None, alpha_pcolor=None
):
    x = np.array(samples[0])
    y = np.array(samples[1])
    xlow, xhigh = xlims
    ylow, yhigh = ylims
    if ls is None:
        ls = linestyle_dict[waveform_model]
    if lw is None:
        lw = linewidth_dict[waveform_model]
    if label is None:
        label = event + " " + label_dict[waveform_model]
    add_samples_to_fig(
        x, xlims, label, alpha_dict[waveform_model],
        ls, color_dict[event], ax1, zorder=zorder, fill=fill_histogram,
        ptype=ptype, linewidth=lw, add_ci=add_ci_list[0]
    )
    add_samples_to_fig(
        y, ylims, label, alpha_dict[waveform_model],
        ls, color_dict[event], ax4, zorder=zorder, right_figure=True, fill=fill_histogram,
        swap_axis=True, ptype=ptype, linewidth=lw, add_ci=add_ci_list[1]
    )
    _range = [[xlow, xhigh], [ylow, yhigh]]
    corner.hist2d(x, y, bins=300, ax=ax3, range=_range, levels=[0.9], color=color_dict[event],
                  plot_datapoints=False, plot_density=plot_density, smooth=7,
                  fill_contours=fill_contours, no_fill_contours=no_fill_contours,
                  contour_kwargs=dict(linestyles=[ls],linewidths=[lw]),
                  pcolor_kwargs=dict(alpha=alpha_pcolor, zorder=zorder - 10))


def combine_samples(waveform_models, outfile):
    data = np.genfromtxt(result_file_map[waveform_models[0]], names=True)
    params = data.dtype.names
    nsamples = len(data)
    dictionary = {
        param: [i[params.index(param)] for i in data] for param in params
    }
    for num, waveform_model in enumerate(waveform_models[1:]):
        data = np.genfromtxt(result_file_map[waveform_model], names=True)
        params = data.dtype.names
        tmp_dict = {
            param: [i[params.index(param)] for i in data] for param in params
        }
        for key in params:
            if key in dictionary.keys():
                dictionary[key] += tmp_dict[key]
    new_dictionary = {}
    for key in dictionary.keys():
        if len(dictionary[key]) != nsamples:
            new_dictionary[key] = dictionary[key]
    
    params = list(new_dictionary.keys())
    samples = np.array([dictionary[param] for param in params])
    np.savetxt(outfile, samples.T, delimiter="\t", header="\t".join(params), comments="")
    return

def create_colormap_from_with_fixed_hue(name, color, N=10, use_saturation=True):
    """Create a linear segmented colormap with fixed hue set by rgb input color.
    Set saturation and value from builtin colormap.

    Parameters:
    `name`: string containing name of the colormap
    `color`: rgb format color that determines the hue.
    `N`: number of colors used in the palette from which the colormap is built.
    """
    from matplotlib.colors import rgb_to_hsv, hsv_to_rgb, hex2color, LinearSegmentedColormap
    import seaborn as sns

    Blues_rgb = sns.color_palette("Blues", 10)
    Blues_hsv = np.array(list(map(rgb_to_hsv, Blues_rgb)))
    h_array, s_array, v_array = Blues_hsv.T
    # Hue is constant, saturation rises monotonically from 0 to 1
    # Value decreases monotonically from 1 to 0.5 with color index in the palette.
    # Value is interpreted as mixture with grayscale: from 0 (black) to 1 (white).

    # To have greater distinction in our palette we also vary the saturation.
    # We need to pick this up as a weighting factor from the input color.

    # Setup array filled with the hue of the current color
    color_hsv = rgb_to_hsv(hex2color(color))
    h_array_fixed = np.ones_like(h_array) * color_hsv[0]
    # Create array of rgb values with fixed hue and varying saturation and value
    if use_saturation:
        s_array *= color_hsv[1]
    color_array = np.array(list(map(hsv_to_rgb,
                                    np.vstack([h_array_fixed, s_array*color_hsv[1], v_array]).T)))
    # Now create the colormap
    return LinearSegmentedColormap.from_list(name, color_array)

